INFINITY_MICROSECONDS = 2 ** 63 - 1

MINUS_INFINITY_MICROSECONDS = -1 * INFINITY_MICROSECONDS - 1
MIN_INT2, MAX_INT2 = -2 ** 15, 2 ** 15
MIN_INT4, MAX_INT4 = -2 ** 31, 2 ** 31
MIN_INT8, MAX_INT8 = -2 ** 63, 2 ** 63


from .pg import POSTGRES_DATA_TYPES


COUNTERPARTS = {
    "postgres": POSTGRES_DATA_TYPES,
}